# Generated by Django 5.0.12 on 2025-09-15 13:45

import django.core.validators
from decimal import Decimal
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wbportfolio', '0088_orderproposal_total_effective_portfolio_contribution'),
    ]

    operations = [
        migrations.AddField(
            model_name='orderproposal',
            name='min_weighting',
            field=models.DecimalField(decimal_places=8, default=Decimal('0'), help_text='The minimum weight allowed for this order proposal ', max_digits=9, verbose_name='Minimum Weight'),
        ),
        migrations.AddField(
            model_name='portfolio',
            name='default_order_proposal_min_order_value',
            field=models.IntegerField(default=0, verbose_name='Default Order Proposal Minimum Order Value'),
        ),
        migrations.AddField(
            model_name='portfolio',
            name='default_order_proposal_min_weighting',
            field=models.DecimalField(decimal_places=8, default=Decimal('0'), max_digits=9,
                                      verbose_name='Default Order Proposal Minimum Weight'),
        ),
        migrations.AddField(
            model_name='portfolio',
            name='default_order_proposal_total_cash_weight',
            field=models.DecimalField(decimal_places=4, default=Decimal('0'), max_digits=5,
                                      verbose_name='Default Order Proposal Total Cash Weight'),
        ),
        migrations.AlterField(
            model_name='orderproposal',
            name='min_weighting',
            field=models.DecimalField(decimal_places=8, default=Decimal('0'),
                                      help_text='The minimum weight allowed for this order proposal ', max_digits=9,
                                      validators=[django.core.validators.MinValueValidator(Decimal('0')),
                                                  django.core.validators.MaxValueValidator(Decimal('1'))],
                                      verbose_name='Minimum Weight'),
        ),
        migrations.AlterField(
            model_name='orderproposal',
            name='total_cash_weight',
            field=models.DecimalField(decimal_places=4, default=Decimal('0'),
                                      help_text='The desired percentage for the cash component. The remaining percentage (100% minus this value) will be allocated to total target weighting. Default is 0%.',
                                      max_digits=5, validators=[django.core.validators.MinValueValidator(Decimal('0')),
                                                                django.core.validators.MaxValueValidator(Decimal('1'))],
                                      verbose_name='Total Cash Weight'),
        ),
        migrations.AlterField(
            model_name='portfolio',
            name='default_order_proposal_min_weighting',
            field=models.DecimalField(decimal_places=8, default=Decimal('0'), max_digits=9,
                                      validators=[django.core.validators.MinValueValidator(Decimal('0')),
                                                  django.core.validators.MaxValueValidator(Decimal('1'))],
                                      verbose_name='Default Order Proposal Minimum Weight'),
        ),
        migrations.AlterField(
            model_name='portfolio',
            name='default_order_proposal_total_cash_weight',
            field=models.DecimalField(decimal_places=4, default=Decimal('0'), max_digits=5,
                                      validators=[django.core.validators.MinValueValidator(Decimal('0')),
                                                  django.core.validators.MaxValueValidator(Decimal('1'))],
                                      verbose_name='Default Order Proposal Total Cash Weight'),
        ),
    ]
