# Generated by Django 5.0.14 on 2025-07-02 13:10

from decimal import Decimal
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wbportfolio', '0077_remove_transaction_currency_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='trade',
            name='drift_factor',
            field=models.DecimalField(decimal_places=6, default=Decimal('1'), help_text='Drift factor to be applied to the previous portfolio weight to get the actual effective weight including daily return', max_digits=16, verbose_name='Drift Factor'),
        ),
        migrations.AlterField(
            model_name='trade',
            name='weighting',
            field=models.DecimalField(decimal_places=8, default=Decimal('0'),
                                      help_text='The weight to be multiplied against the target', max_digits=9,
                                      verbose_name='Weight'),
        ),
    ]
