# Generated by Django 4.1.9 on 2023-06-23 07:48

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbfdm", "0001_initial"),
        ("wbportfolio", "0002_product_default_sub_account_squashed_0039_alter_assetallocation_company_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="instrument",
            name="financial_instrument",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="pms_instruments",
                to="wbfdm.instrument",
                verbose_name="Financial Instrument",
            ),
        ),
    ]
